package comptes;
import java.util.Vector;
import java.util.Date;
import java.text.DecimalFormat;

/** Modlisation d'un compte bancaire. */
public class Compte {

 private String titulaire; // attributs d'instance
 private int numero;
 private double solde;
 private Vector releve;
 private static int nbComptes=0;  // attribut de classe
 
 private double decouvertAutorise; // attribut d'instance
 private static double decouvertAutoriseParDefaut =100; // attribut de classe
 
 // constructeurs
 /** Cre un compte pour le titulaire spcifi. */
 public Compte(String nomTitulaire) {
   Compte.nbComptes++; // ou this.nbComptes++; // ou nbComptes++;
   this.numero = nbComptes;
   this.titulaire = nomTitulaire;
   this.releve = new Vector();
   this.decouvertAutorise = Compte.decouvertAutoriseParDefaut;
   // ou this.decouvertAutorise = this.decouvertAutoriseParDefaut;
   // ou this.decouvertAutorise = decouvertAutoriseParDefaut;
   // ou setDecouvertAutorise(decouvertAutoriseParDefaut);
   // releve et decouvertAutorise peuvent tre initialiss avec les dclarations
   }
 
 /** constante pour le paiement en espces . */
 public static final int ESPECES = 0; // constantes
 /** constante pour le paiement par chque */
 public static final int CHEQUE = 1;
 /** constante pour le paiement par carte bancaire */
 public static final int CB = 2;
 /** constante pour la transaction par virement */
 public static final int VIREMENT = 3;

  /** Cre un compte pour le titulaire spcifi 
   *          avec le versement inital,
   *               le moyen de paiement,
   *               le dcouvert autoris spcifis. */
 public Compte(String nomTitulaire, double versementInitial, int moyenPaiement,
               double decouvertAutorise) {
   this(nomTitulaire);
   this.crediter(versementInitial, moyenPaiement);
   }
 /** Cre un compte pour le titulaire spcifi 
   *          avec le versement inital,
   *               le moyen de paiement,
   *               le dcouvert autoris par dfaut. */
 public Compte(String nomTitulaire,double versementInitial,int moyenPaiement) {
   this(nomTitulaire, versementInitial, moyenPaiement,
                                            Compte.decouvertAutoriseParDefaut);
   }

 // attributs de classe non modifiables
 private static final String[] lesMoyensPaiement =
                              {"espces ", "chque  ", "CB      ", "virement"};
 protected static final DecimalFormat df = new DecimalFormat("###0.00");
 
  /** Crdite ce compte du montant spcifi, 
   *     avec le moyen de paiement spcifi. */
 public void crediter(double montant, int moyenPaiement) {
   if (montant > 0) this.enregistrer(montant, moyenPaiement);
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - crdit non effectu");}
 
 /** Enregistre pour ce compte le montant (sign) spcifi,
  *                     avec le moyen de paiement spcifi. */
 public void enregistrer(double montant, int moyenPaiement) {
   this.solde += montant;
   String date = new Date().toString();
   date = date.substring(4,16)+date.substring(23,28);
   this.releve.addElement(date+" "+lesMoyensPaiement[moyenPaiement]+
               ((montant >0)?" crdit +":" dbit  ")+Compte.df.format(montant));
   }

 /** Dbite, si possible, ce compte du montant spcifi,
  *                  avec le moyen de paiement spcifi. */
 public void debiter(double montant, int moyenPaiement) {
   if (montant > 0)
     if (this.isDebitPossible(montant))this.enregistrer(-montant,moyenPaiement);
     else System.out.println("dbit non autoris, solde insuffisant");
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - dbit non effectu");}
 /** Teste si le dbit spcifi est possible pour ce compte. */
 public boolean isDebitPossible(double montant) {
                           return montant < this.solde+this.decouvertAutorise;}
 /** Vire, si possible, le montant spcifi de ce compte 
  *                              vers le compte spcifi. */
 public void virer(double montant, Compte c) {
   if (montant >0)
      if (isDebitPossible(montant)) {
         this.debiter(montant, Compte.VIREMENT);
         c.crediter(montant, Compte.VIREMENT);
         }
      else System.out.println("dbit non autoris");
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - virement non effectu");
   }

 /** Renvoie le numro de ce compte. */
 public int getNumero() {return this.numero;}
 /**  Renvoie le titulaire de ce compte. */
 public String getTitulaire() {return this.titulaire;}
 /**  Renvoie le solde de ce compte. */
 public double getSolde() {return this.solde;}
 /**  Renvoie le relev de ce compte. */
 public String getReleve() {
  StringBuffer sb = new StringBuffer(this.toString()+"relev : \n");
  for (int i=0; i<this.releve.size();i++)
            sb.append(this.releve.elementAt(i)+"\n");
  return sb.toString();
  }

 /** Renvoie le RIB de ce compte. */
 public String RIB() {
   return "compte numro : "+ this.numero + "\ntitulaire : " + this.titulaire;}
 /** Renvoie une reprsentation sous forme de chaine de ce compte */
 public String toString() {
   String s = "\n==================\n"
         + this.RIB() + "\nsolde : " + Compte.df.format(this.solde)
         + "\ndcouvert autoris : " + Compte.df.format(this.decouvertAutorise);
   if (this.solde < 0) s += "\nvous tes en dcouvert";
   s+="\n==================\n";
   return s;
   }

 /** Renvoie le dcouvert autoris de ce compte. */
 public double getDecouvertAutorise() {return this.decouvertAutorise;}
 /** Remplace le dcouvert autoris de ce compte 
  *           par le dcouvert autoris spcifi. */
 public void setDecouvertAutorise(double decouvertAutorise) {
   if (decouvertAutorise >0) this.decouvertAutorise = decouvertAutorise;}
 /** Renvoie le dcouvert autoris par dfaut (pour les nouveaux comptes). */
 public static double getDecouvertAutoriseParDefaut() {
                                   return Compte.decouvertAutoriseParDefaut;}
  /** Remplace le dcouvert autoris par dfaut 
   *  par le dcouvert autoris par dfaut spcifi. */
  public static void setDecouvertAutoriseParDefaut(double decouvertAutoriseParDefaut) {
   if (decouvertAutoriseParDefaut >0)
                 Compte.decouvertAutoriseParDefaut = decouvertAutoriseParDefaut;}

}
