package comptes;
import java.util.Vector;
import java.util.Date;
import java.text.DecimalFormat;

import java.io.*;

/** Modlisation d'un compte bancaire, avec sauvegarde et chargement,
 * seules les nouvelles mthodes sont documentes. */
public class CompteAvecSauvegarde implements Serializable {

 private String titulaire; // attributs d'instance
 private int numero;
 private double solde;
 private Vector releve;
 private static int nbComptes=0;  // attribut de classe
 
 private double decouvertAutorise; // attribut d'instance
 private static double decouvertAutoriseParDefaut =100; // attribut de classe
 
 // constructeurs
 public CompteAvecSauvegarde(String nomTitulaire) {
   CompteAvecSauvegarde.nbComptes++; // ou this.nbComptes++; // ou nbComptes++;
   this.numero = nbComptes;
   this.titulaire = nomTitulaire;
   this.releve = new Vector();
   this.decouvertAutorise = CompteAvecSauvegarde.decouvertAutoriseParDefaut;
   // ou this.decouvertAutorise = this.decouvertAutoriseParDefaut;
   // ou this.decouvertAutorise = decouvertAutoriseParDefaut;
   // ou setDecouvertAutorise(decouvertAutoriseParDefaut);
   // releve et decouvertAutorise peuvent tre initialiss avec les dclarations
   }
 
 public static final int ESPECES = 0; // constantes
 public static final int CHEQUE = 1;
 public static final int CB = 2;
 public static final int VIREMENT = 3;

 public CompteAvecSauvegarde(String nomTitulaire, double versementInitial, 
                             int moyenPaiement, double decouvertAutorise) {
   this(nomTitulaire);
   this.crediter(versementInitial, moyenPaiement);
   }
 public CompteAvecSauvegarde(String nomTitulaire,double versementInitial,
                                                 int moyenPaiement) {
   this(nomTitulaire, versementInitial, moyenPaiement,
                               CompteAvecSauvegarde.decouvertAutoriseParDefaut);
   }

 // attributs de classe non modifiables
 private static final String[] lesMoyensPaiement =
                              {"espces ", "chque  ", "CB      ", "virement"};
 protected static final DecimalFormat df = new DecimalFormat("###0.00");
 
 public void crediter(double montant, int moyenPaiement) {
   if (montant > 0) this.enregistrer(montant, moyenPaiement);
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - crdit non effectu");}
 
 public void enregistrer(double montant, int moyenPaiement) {
   this.solde += montant;
   String date = new Date().toString();
   date = date.substring(4,16)+date.substring(23,28);
   this.releve.addElement(date+" "+lesMoyensPaiement[moyenPaiement]+
                          ((montant >0)?" crdit +":" dbit  ")+
                          CompteAvecSauvegarde.df.format(montant));
   }

 public void debiter(double montant, int moyenPaiement) {
   if (montant > 0)
     if (this.isDebitPossible(montant))this.enregistrer(-montant,moyenPaiement);
     else System.out.println("dbit non autoris, solde insuffisant");
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - dbit non effectu");}
 public boolean isDebitPossible(double montant) {
                           return montant < this.solde+this.decouvertAutorise;}
 public void virer(double montant, CompteAvecSauvegarde c) {
   if (montant >0)
      if (isDebitPossible(montant)) {
         this.debiter(montant, CompteAvecSauvegarde.VIREMENT);
         c.crediter(montant, CompteAvecSauvegarde.VIREMENT);
         }
      else System.out.println("dbit non autoris");
   else System.out.println(
          "erreur : vous avez donn un montant ngatif - virement non effectu");
   }

 public int getNumero() {return this.numero;}
 public String getTitulaire() {return this.titulaire;}
 public double getSolde() {return this.solde;}
 public String getReleve() {
  StringBuffer sb = new StringBuffer(this.toString()+"relev : \n");
  for (int i=0; i<this.releve.size();i++)
            sb.append(this.releve.elementAt(i)+"\n");
  return sb.toString();
  }

 public String RIB() {
   return "compte numro : "+ this.numero + "\ntitulaire : " + this.titulaire;}
 public String toString() {
   String s = "\n==================\n"
            + this.RIB() + "\nsolde : " + CompteAvecSauvegarde.df.format(this.solde)
            + "\ndcouvert autoris : " + CompteAvecSauvegarde.df.format(this.decouvertAutorise);
   if (this.solde < 0) s += "\nvous tes en dcouvert";
   s+="\n==================\n";
   return s;
   }

 public double getDecouvertAutorise() {return this.decouvertAutorise;}
 public void setDecouvertAutorise(double decouvertAutorise) {
   if (decouvertAutorise >0) this.decouvertAutorise = decouvertAutorise;}
 public static double getDecouvertAutoriseParDefaut() {
                                   return CompteAvecSauvegarde.decouvertAutoriseParDefaut;}
 public static void setDecouvertAutoriseParDefaut(double decouvertAutoriseParDefaut) {
   if (decouvertAutoriseParDefaut >0)
     CompteAvecSauvegarde.decouvertAutoriseParDefaut = 
                                           decouvertAutoriseParDefaut;}

  /** Enregistre ce compte dans un fichier de nom le titulaire du compte,
   *  ainsi que le nombre de comptes ouverts dans un fichier de nom "nbComptes".
   *  En cas d'chec, affiche un message. */
  public void sauvegarder(){
    ObjectOutputStream flotEcriture=null;
    File fichier = new File(getTitulaire());
    try{ flotEcriture = new ObjectOutputStream(new FileOutputStream(fichier));
         flotEcriture.writeObject(this);
         System.out.println("compte sauvegard");
         }
    catch (Exception e)
                {System.out.println("Sauvegarde non effectue");}
    finally {try {flotEcriture.close();}
             catch (Exception  e){}}


    DataOutputStream donneeEcriture = null;
    try { donneeEcriture = 
                     new DataOutputStream(new FileOutputStream("nbComptes"));
          donneeEcriture.writeInt(nbComptes);
        }
    catch (Exception e) {System.out.println(e);}
    finally {try {donneeEcriture.close();}
             catch (Exception  e){}}
    }

  /** S'il existe un fichier de nom 'titulaire', 
   *        renvoie le compte enregistr dans celui-ci
   *        sinon cre un nouveau compte pour 'titulaire'.
   *  S'il existe un fichier de nom "nbComptes" contenant un entier plus grand
   *  que la constante nbComptes, nbComptes sera remplac par cette valeur.
   *  En cas d'chec affiche un message. */
  public static CompteAvecSauvegarde charger(String titulaire)  {
    File fichier = new File("nbComptes");
    if (fichier.exists()) {
      DataInputStream donneeLecture = null;
      try{ donneeLecture = new DataInputStream(new FileInputStream(fichier));
           int nb = donneeLecture.readInt();
           nbComptes = Math.max(nbComptes,nb);
           donneeLecture.close();
           System.out.println("nbComptes = "+nbComptes);
           }
      catch (Exception e) {System.out.println(e);}
      finally {try   {donneeLecture.close();}
               catch (Exception  e){}; }
      }
    
    fichier = new File(titulaire);
    if (!fichier.exists()) { System.out.println("fichier inexistant");
                             return new CompteAvecSauvegarde(titulaire);}
    ObjectInputStream flotLecture = null;
    try { flotLecture = new ObjectInputStream(new FileInputStream(fichier));
          System.out.println("chargement du compte de "+titulaire);
          return (CompteAvecSauvegarde)flotLecture.readObject();
          }
    catch (Exception e){System.out.println(e+"\ncompte non charg"); }
    finally {try   {flotLecture.close();}
             catch (Exception  e){}; }
    return null;

    }

} 
