package equations;

public abstract class Nombre {
 abstract Nombre somme(Nombre A);
 abstract Nombre produit(Nombre A);
 protected Nombre carre() { return this.produit(this);}
 abstract Nombre racineCarree() ;
}

class Reel extends Nombre {
  protected double val;
  public Reel(double r) { this.val=r;}
  // public Reel(Double R) {this.val= R.doubleValue();}
  public Nombre somme(Nombre A) {
    if (A instanceof Reel) return new Reel(this.val+((Reel)A).val);
    else if (A instanceof Complexe) return A.somme(new Complexe(this.val,0));
    else return null;
    }
  public Nombre produit(Nombre A) {
    if (A instanceof Reel) return new Reel(this.val*((Reel)A).val);
    else if (A instanceof Complexe) return A.produit(new Complexe(this.val,0));
    else return null;
    }
  public Nombre racineCarree () {
    if (this.val >= 0) return new Reel(Math.sqrt(this.val)) ;
    else return new Complexe(0,Math.sqrt(-this.val));
    }
  public String toString() {return Double.toString(this.val);}
  }

class Complexe extends Nombre {
  private double reel;
  private double imag;
  public Complexe(double x, double y) {this.reel=x; this.imag=y;}
  public Nombre somme(Nombre A) {
    if (A instanceof Reel) return A.somme(this);
    else if (A instanceof Complexe)
        return new Complexe(this.reel+((Complexe)A).reel,
                            this.imag+((Complexe)A).imag);
    else return null;
    }
  public Nombre produit(Nombre A) {
    if (A instanceof Reel) return A.produit(this);
    else if (A instanceof Complexe)
      return
       new Complexe(this.reel*((Complexe)A).reel - this.imag*((Complexe)A).imag,
                    this.reel*((Complexe)A).imag + this.imag*((Complexe)A).reel
                          );
    else return null;
    }
  public Nombre racineCarree() {
    if (this.imag == 0) return new Reel(this.reel).racineCarree();
    Equation e = new Equation(new Reel(-this.reel),
                 new Reel(-this.imag*this.imag/4));
    Nombre x2= e.racine1();
    double x = Math.sqrt(((Reel)x2).val);
    double y = this.imag/x/2;
    return new Complexe(x,y);
    }
  public String toString() {
    double a = this.reel;
    double b = this.imag;
    if (a==0 && b==0) return("0");
    if (a!=0 && b==0) return(""+a);
    if (a==0 && b>0) return("i "+b);
    String signei;
    String res="(";
    if (a!=0) res+=a;
    if (b>0) signei=" + i "; else {signei=" - i "; b=-b;};
    return(res+signei+b+")");
    }
  }

