package gestion;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.io.*;

/** Fentre permettant, pour un ensemble de matires donn, de
 *    - saisir les donnes relatives  un tudiant (identit, groupe, notes)
 *    - gnrer son relev de notes
 *    - mmoriser ces donnes
 *    - calculer des moyennes, par tudiant et par matire
 */
public class Fenetre extends Frame implements ActionListener {
  private Groupe groupe;
  // identite (question 1)
  private TextField tfNom =new TextField(10);
  public TextField tfPrenom =new TextField(10);
  private String nom, prenom;
  private TextField tfAnnee =new TextField(10);
  private int age;
 // matires et notes
  private Matiere[] tabMatieres;
  private PanelMatiere[] tabPanel;
  private double[] notes;
  private boolean complet;  // sera gal  true ssi toutes les donnes
               // (sauf ventuellement le prnom) ont t correctement remplies
  private String releve;
  // boutons (question 1)
  private Button bValider = new Button("valider");
  private Button bEnregistrer = new Button("enregistrer");
  private Button bTerminer = new Button("terminer");
  private Button bQuitter = new Button("quitter");

  // zone de texte  (question 1)
  private TextArea ta = new TextArea(22,10);

 /** cre une fentre de titre le nom du 'groupe' pour la lecture des donnes
  *  relatives aux tudiants de ce groupe et  leurs notes dans les matires de
  *  'tabMatieres'
  */
  public Fenetre(Groupe groupe, Matiere[] tabMatieres) {
    super(groupe.getNom());
    this.groupe=groupe;
    this.tabMatieres= tabMatieres;
    this.notes= new double[this.tabMatieres.length];
    this.setLayout(new BorderLayout());
    Panel questions= new Panel(new GridLayout(3,1));
    this.add("North", questions);
       // identite
       Panel identite = new Panel(new FlowLayout()); questions.add(identite);
          identite.add(new Label("nom"));identite.add(tfNom);
          identite.add(new Label("prenom"));identite.add(tfPrenom);
          identite.add(new Label("anne de naissance")); identite.add(tfAnnee);
       // notes
       Panel pNotes = new Panel(new FlowLayout()); questions.add(pNotes);
       tabPanel = new PanelMatiere[tabMatieres.length];
       for (int i=0; i<tabMatieres.length;i++) {
         tabPanel[i]=new PanelMatiere(tabMatieres[i]);
         pNotes.add(tabPanel[i]);
         }
       // boutons
       Panel boutons = new Panel(new FlowLayout()); questions.add(boutons); 
       boutons.add(bValider); boutons.add(bEnregistrer);
       boutons.add(bTerminer); boutons.add(bQuitter);
    this.add("Center",ta);
    bValider.addActionListener(this);
    bQuitter.addActionListener(this);
    bEnregistrer.addActionListener(this);
    bTerminer.addActionListener(this);
    this.addWindowListener(new GestionnaireExit());
    }

  /** au click sur le bouton 'valider'
   *    - lit les donnes correspondant aux diffrents champs, construit et
   *      affiche le relev de notes dans la zone de texte  (question 1)
   *    - pour chaque donne (sauf prnom) incomplte ou incorrecte,
   *      affiche un message adquat et continue  (question 4)
   *    (on ne cre pas d'instance d'tudiant  cette tape)
   *  au click sur le bouton 'enregistrer'
   *    - si les donnes sont compltes, enregistre l'tudiant et les notes,
   *      enregistre le relev de notes dans un fichier dont le nom est
   *      constitu du nom et du prnom (vide si absent) de l'tudiant
   *      et affiche un message dasn la zone de texte (question 2)
   *    - si les donnes sont incompltes, affiche seulement un message dans la
   *      zone de texte (question 4)
   *  au click sur le bouton 'terminer'
   *    - affiche dans la zone de texte les notes finales de tous les tudiants
   *      du groupe et la moyenne de ces notes, ainsi que les moyennes des notes
   *      de chaque matire (question 3)
   *  au click sur le bouton 'quitter' arrte le programme
   */
  public void actionPerformed(ActionEvent e) {
    DecimalFormat df = new DecimalFormat("0.##");

    if (e.getSource()== bValider) {                     // valider
      this.complet=true;
      // identite
      this.nom = tfNom.getText();
      this.prenom = tfPrenom.getText();
      releve = "Nom : "+nom+" "+"          Prenom : " + prenom;
      if (nom.equals("")) {releve +="\n(nom absent)";this.complet=false;}
      // naissance
      try {
      String annee = tfAnnee.getText();
      age = 2003 - Integer.parseInt(annee);
      releve += "        age : "+age+" ans";
      } catch(Exception ex){
                    releve += "\nanne de naissance inexistante ou incorrecte";
                    this.complet=false;}
      // notes
      releve += "\n\nrelev de notes :";
       try {
       double somme =0;
       for (int i=0; i<tabMatieres.length;i++) {
        String nomMatiere =tabMatieres[i].getNom();
        if (tabMatieres[i] instanceof MatiereOptionnelle) {
          Checkbox cb = tabPanel[i].getCbg().getSelectedCheckbox();
          if (cb == null) throw new Exception ("\noption de "
                                   +tabMatieres[i].getNom()+" non coche");
          nomMatiere = cb.getLabel();
          }
        releve += "\n   "+nomMatiere+" : ";
        String s = (tabPanel[i].getTfnote()).getText();
        if (s.equals("")) throw new Exception("\nnote(s) absente(s)");
        double note = Double.parseDouble(s);
        if (note <0 || note >20) throw new Exception(
                           "\nles notes doivent tre comprises entre 0 et 20");
        this.notes[i]=note;
        releve += df.format(note);
        somme += note;
        }
      double moyenne = somme/tabMatieres.length;
      String dfMoyenne = df.format(moyenne);
      releve += "\n\nnote finale (moyenne) : "+dfMoyenne;
      } catch(Exception ex){releve += ex.getMessage();this.complet=false;}
      // message
      ta.setText(releve);
      }

    if (e.getSource()== bEnregistrer) {                 // enregistrer
      if (this.complet) {
        Etudiant etu = Etudiant.enregistrer(this.groupe, this.nom, this.prenom,
                          this.age, this.tabMatieres, this.notes, this.releve);
        ta.append("\n\ndonnes mmorises et relev de notes enregistr "
                  +"dans le fichier "+ this.nom+"_"+this.prenom);
        System.out.println("\n"+this.releve);
        try {
        PrintWriter out =
                     new PrintWriter(new FileWriter(this.nom+"_"+this.prenom));
        out.write(this.releve);
        out.close();
        } catch(Exception ex){}
        }
       else ta.append("\n\n*************************************************"
                    +"\n* donnes erronnes ou incompltes non enregistres *"
                    +"\n**************************************************");
    }

    if (e.getSource()== bTerminer) {                    // terminer
      System.out.println(this.groupe);
      ta.append(this.groupe.toString());
      String s = "\n\n==========" +" moyennes des "
                 +this.tabMatieres[0].getVecNotes().size()
                 +" notes de chaque matire"+" =========";
      for (int i=0;i<tabMatieres.length;i++) {
         Matiere mat = this.tabMatieres[i];
         // s += "\n("+mat.getVecNotes().size()+ " notes) ";
         s += "\n"+mat.getNom()+" : "+df.format(mat.moyenne());
         }
      System.out.println(s);
      ta.append(s);
      }
    if (e.getSource()== bQuitter) System.exit(0);       // quitter
    }
}
