package ab;                                                        // juin 04 //
import iv.FileAttente;

/** Implmentation d'arbres binaires et de plusieurs reprsentations. */
public class ArbreBinaire {
  /*private*/ protected Object racine; // protected pour numrations
  /*private*/ protected ArbreBinaire gauche,droit;// sous-arbres gauche et droit

  /** Cre un arbre binaire compos d'un seul noeud (cad une feuille). */
  public ArbreBinaire(Object racine) { this.racine=racine;}

  /** Cre un arbre binaire compos de la racine spcifie et des deux
   *  sous-arbres spcifis */
  public ArbreBinaire(Object racine, ArbreBinaire gauche, ArbreBinaire droit) {
    this(racine);
    this.gauche=gauche;
    this.droit=droit;
    }

   /** Cre un arbre binaire compos de la racine spcifie et du sous-arbre
    *  spcifi (qui sera les sous-arbre droit). */
  public ArbreBinaire(Object racine, ArbreBinaire droit) {
   this(racine, null,droit);
   }

   /** Cre un arbre binaire vide de la hauteur spcifie. */                 //
  public ArbreBinaire(int n) {                                                //
   this("");                                                                  //
   if (n>1) { this.racine="";                                                 //
              this.gauche = new ArbreBinaire(n - 1);                          //
              this.droit = new ArbreBinaire(n-1);                             //
              }                                                               //
  }                                                                           //


  /** Teste si cet Arbre binaire est une feuille. */
  public boolean feuille() {
    return this.gauche == null && this.droit == null ;
    }

  /** Renvoie une reprsentation indente de cet arbre binaire, c'est--dire :
   *  pour chaque sous-arbre il y a passage  la ligne et une indentation
   *  proportionnelle  la profondeur de ce sous-arbre. */
   public String toString() { return this.toString("");}
   /** Renvoie une reprsentation indente de cet arbre binaire indente de plus
    *  de la chaine spcifie. */
   public String toString(String indent) {
      String s= indent;
      s+=this.racine;
      indent += "   ";
      if (!(this.gauche==null)) s+="\n"+this.gauche.toString(indent);
      if (!(this.droit==null)) s+="\n"+this.droit.toString(indent);
      return s;
      }

   /** Renvoie une reprsentation prfixe de cet arbre binaire,
    *  c'est--dire dans l'ordre :
    *  racine puis sous-arbres gauche puis droit prfixs. */
   public String formePrefixe() {
     String s = ""+this.racine;
     if (this.gauche != null) s+=" "+ this.gauche.formePrefixe();
     if (this.droit != null) s+= " "+this.droit.formePrefixe();
     return s;
     }

  /** Renvoie une reprsentation postfixe de cet arbre binaire,
  *  c'est--dire dans l'ordre :
  *  sous-arbres gauche puis droit postfixs puis racine. */
  public String formePostfixe() {
    String s = "";
    if (this.gauche != null) s+= this.gauche.formePostfixe();
    if (this.droit != null) s+= this.droit.formePostfixe();
    s+=" "+this.racine;
    return s;
    }

   /** Renvoie une reprsentation infixe de cet arbre binaire,
    *  c'est--dire dans l'ordre :
    *  sous-arbre gauche infixe puis racine puis sous-arbre droit infixe. */
   public String formeInfixe() {
     String s="";
     if (this.gauche != null) s+= this.gauche.formeInfixe();
     s+=" "+this.racine;
     if (this.droit != null) s+= this.droit.formeInfixe();
     return s ;
     }

    /** Renvoie une reprsentation infixe de cet arbre binaire dans laquelle
     *  chaque sous-arbre est entour de parenthses sauf les feuilles. */
    public String formeInfixePar() {
      boolean par = false;
      if (! this.feuille()) par = true;
      String s="";
      if (par) s+= " (";
      if (this.gauche != null) s+= this.gauche.formeInfixePar();
      s+=" "+this.racine;
      if (this.droit != null) s+= this.droit.formeInfixePar();
      if (par) s+=" )";
      return s ;
      }

    /** Renvoie une reprsentation par niveau de cet arbre binaire,           //
     *  c'est--dire dans l'ordre :                                           //
     *  racine puis noeuds de niveau 1, puis 2, etc ... . */                  //
    public String formeParNiveau() {                                          //
      FileAttente f = new FileAttente();                                      //
      f.rentrer(this);                                                        //
      String s="";                                                            //
      while (! f.vide()) {                                                    //
        ArbreBinaire a = (ArbreBinaire) f.premier();                          //
        s+= a.racine+" ";                                                     //
        f.sortir();                                                           //
        if (a.gauche != null) f.rentrer(a.gauche);                            //
        if (a.droit != null) f.rentrer(a.droit);                              //
        }                                                                     //
      return s;                                                               //
      }                                                                       //

    /** Renvoie une reprsentation de cet arbre binaire par niveaux spars   //
     *  par la marque spcifie. */                                           //
    public String formeParNiveauxSepares(Object marque) {                     //
      FileAttente f = new FileAttente();                                      //
      f.rentrer(marque);                                                      //
      f.rentrer(this);                                                        //
      String s="";                                                            //
      while (! f.vide()) {                                                    //
        Object o = f.premier();                                               //
        f.sortir();                                                           //
        if (f.vide()) s+= marque;                                             //
        else if (o instanceof ArbreBinaire) {                                 //
                ArbreBinaire a = (ArbreBinaire) o;                            //
                if (a.gauche != null) f.rentrer(a.gauche);                    //
                if (a.droit != null) f.rentrer(a.droit);                      //
                s+= a.racine+" ";                                             //
                }                                                             //
              else {f.rentrer(o);s+=o;}                                       //
              }                                                               //
      return s;                                                               //
      }                                                                       //
               

      //
      // Reprsentations
      //                  pr/post/infixes/par niveau
      // utilisant des numrations
      //


    public String formePrefixeEn() {
      String s = "";
      for (EnumerationPrefixe e = new EnumerationPrefixe(this);
                                           e.hasMoreElements();)
         s+=e.nextElement()+" ";
         return s;
         }



     public String formePostfixeEn() {
       String s = "";
       for (EnumerationPostfixe e = new EnumerationPostfixe(this);
                                              e.hasMoreElements();)
         s+=e.nextElement()+" ";
         return s;
         }
     public String formeInfixeEn() {
       String s="";
       for (EnumerationInfixe e = new EnumerationInfixe(this);
                                          e.hasMoreElements();)
         s+=e.nextElement()+" ";
         return s ;
         }
     public String formeInfixeParEn() {
       String s="";
       for (EnumerationInfixePar e = new EnumerationInfixePar(this);
                                                e.hasMoreElements();)
         s+=e.nextElement()+" ";
         return s ;
         }


      public String formeParNiveauEn() {                                      //
        String s="";                                                          //
        for (EnumerationParNiveau e = new EnumerationParNiveau(this);         //
                                                e.hasMoreElements();)         //
         s+=e.nextElement()+" ";                                              //
         return s ;                                                           //
      }                                                                       //

      public String formeParNiveauxSeparesEn(Object marque) {                 //
        String s="";                                                          //
        for (EnumerationParNiveauxSepares e =                                 //
                           new EnumerationParNiveauxSepares(this,marque);     //
                   e.hasMoreElements();)                                      //
         s+=" "+e.nextElement();                                              //
         return s ;                                                           //
      }                                                                       //

}





